function [a,markI,lattnum] = check_iterative_MR1L(I,Ms,zs)
%CHECK_ITERATIVE_MR1L Checks whether a set of frequencies can be reconstructed with given multiple rank-1 lattices.
%
%  function [a,markI,lattnum] = check_iterative_MR1L(I,Ms,zs)
%
%   
%
%   input:
%       I:  frequency index set (matrix |I| x d)
%       Ms: vector of genereting vectors in 'decreasing' order
%       zs: corresponding generating vectors 'decreasing' order w.r.t. Ms
%           (matrix length(Ms) x d)
%
%   output:
%       a:       boolean that states wether (zs,Ms) is a reco MR1L for I
%       markI:   boolean vector, markI(j)=1 => the Fourier coefficient with 
%                frequency I(j,:) can be reconstructed using this MR1L
%                if a=true should be vector of ones
%                if a=false there should be at least one zero
%       lattnum: false, if a=false
%                integer, stating that Ms(1:lattnum) and zs(1:lattnum,:)
%                is already a reconstructing MR1L


% checks
if(size(I,2)~=size(zs,2))
    error('dimension of frequency vectors and generating vectors do not fit');
end

if(size(zs,1)~=length(Ms) || size(zs,1)~=prod(size(Ms)))
    error('number of lattice sizes M and generating vectors z do not fit');
end

a=false;
lattnum=false;
markI=false(size(I,1),1);
ind6=1:size(I,1);
for j=1:size(zs,1)

    zs(j,:)=mod(zs(j,:),Ms(j));
    tmp=mod(I(ind6,:).*zs(j,:),Ms(j));
    tmp1=mod(sum(tmp,2),Ms(j));
 
    [tmp3,ind3]=sort(tmp1);
    tmp4=tmp3(2:end)-tmp3(1:end-1);
    if(tmp4(1)~=0)
        markI(ind6(ind3(1)))=true;
    end
    if(tmp4(end)~=0)
        markI(ind6(ind3(end)))=true;
    end
    tmp5=tmp4(1:end-1).*tmp4(2:end);
    ind5=find(tmp5~=0)+1;
    markI(ind6(ind3(ind5)))=true(length(ind5),1);
    
    ind6=find(markI==0);
    
    if(length(ind6)==0)
        a=true;
        lattnum=j;
        break;
    end
end

