function [Ms,hred_array,hrem_array] = deterministic_mr1l_get_lattice_sizes_largeMstartS(freq,r1l_z_array_cell,r1l_M)
%DETERMINISTIC_MR1L_GET_LATTICE_SIZES Computes multiple rank-1 lattices by Theorem 2.8.

if size(freq,1) == 1
  Ms = 2;
  hred_array = 1;
  hrem_array = 0;
  return;
end

freq = int64(freq);
if iscell(r1l_z_array_cell)
  for t = 1:length(r1l_z_array_cell)
    r1l_z_array_cell{t} = int64(r1l_z_array_cell{t});
  end
else
  r1l_z_array_cell = int64(r1l_z_array_cell);
end
N_I = max(max(freq,[],1)-min(freq,[],1)+1);
[s,d] = size(freq);

r1l_z_output = [];

if ~iscell(r1l_z_array_cell) && (d * double(N_I) * double(r1l_M) < 2^63)
  freq_times_z_precomputed = freq(:,1) * r1l_z_array_cell(1);
  for t = 2:d
    freq_times_z_precomputed = freq_times_z_precomputed + freq(:,t) * r1l_z_array_cell(t);
  end
else
  freq_times_z_precomputed = [];
end

% if d * double(N_I) * double(r1l_M) > 2^63
%   error('quantities too large');
% end

% htimesz = sum(freq .* r1l_z, 2);
% htimesz = freq(:,1) * r1l_z(1);
% for t = 2:d
%   if double(max(freq(:,t))) > 2^63 / d / double(r1l_z(t))
%     error('quantities too large');
%   end
%   htimesz = htimesz + freq(:,t) * r1l_z(t);
% end
% 
% if length(unique(mod(htimesz,r1l_M))) ~= size(freq,1)
%   error('rank-1 lattice is not reconstructing');
% end

P_upper_bound = floor( 5*s * log(double(d*N_I*r1l_M))/log(s) * log(4*s*log(double(d*N_I*r1l_M))/log(s)) );
max_P = min(P_upper_bound,2*r1l_M);
if max_P > 2e9
  warning('Upper bound on P too large => restricting to 2e9. Search may fail!');
  max_P = 2e9;
end
max_P = int64(max_P);
primes_array = primes(max_P);

h1 = freq;

% P = 2;
Ms = [];
hred_array = [];
hrem_array = [];
while (size(h1,1) >= 2)
  s = size(h1,1);
  P_upper_bound = floor( 5*s * log(double(d*N_I*r1l_M))/log(s) * log(4*s*log(double(d*N_I*r1l_M))/log(s)) );
  cur_primes_array = primes_array(primes_array <= P_upper_bound);
  cur_primes_array = cur_primes_array(cur_primes_array >= size(h1,1)/2);
  ind = find(cur_primes_array >= s, 1);
  cur_primes_array = [cur_primes_array(ind:end),cur_primes_array(1:ind-1)];
  for iP = 1:length(cur_primes_array)
    P = int64(cur_primes_array(iP));

%       if iscell(r1l_z_array_cell)
%         r1l_z = int64(ones(1,d));
%         for t = 1:d
%           for ii = 1:length(r1l_z_array_cell{t})
%             r1l_z(t) = mod(r1l_z(t) * mod(r1l_z_array_cell{t}(ii),P), P);
%           end
%         end
%       else
%         r1l_z = mod(r1l_z_array_cell,P);
%       end
%           h1timesz = mod(mod(h1(:,1),P) * r1l_z(1),P);
%           for t = 2:d
%             h1timesz = h1timesz + mod(mod(h1(:,t),P) * r1l_z(t),P);
%           end
%           
%             y=mod(h1timesz,P);
      if ~isempty(freq_times_z_precomputed)
        y = mod(freq_times_z_precomputed,P);
        r1l_z = mod(r1l_z_array_cell,P);
      else
        if iscell(r1l_z_array_cell)
          r1l_z = int64(ones(1,d));
          for t = 1:d
            for ii = 1:length(r1l_z_array_cell{t})
              r1l_z(t) = mod(r1l_z(t) * mod(r1l_z_array_cell{t}(ii),P), P);
            end
          end
        else
          r1l_z = mod(r1l_z_array_cell,P);
        end
        freq_times_z = mod(mod(h1(:,1),P) * r1l_z(1),P);
        for t = 2:d
          freq_times_z = freq_times_z + mod(mod(h1(:,t),P) * r1l_z(t),P);
        end
        y = mod(freq_times_z,P);
      end

            
            [y,ind]=sort(y);
            [y,ind2]=setdiff(y,y(y(2:end)-y(1:end-1)==0));
            hred = length(y);
            if hred >= size(h1,1) / 2
%               h1 = setdiff(h1,h1(ind(ind2),:),'rows');
              h1(ind(ind2),:) = [];
              Ms = [Ms; P];
              hred_array = [hred_array; hred];
              hrem_array = [hrem_array; size(h1,1)];
              r1l_z_output = [r1l_z_output; r1l_z];
              if ~isempty(freq_times_z_precomputed)
                freq_times_z_precomputed(ind(ind2),:) = [];
              end
              break;
            end
            if iP == length(cur_primes_array)
              error('lattice not found');
            end
  end
end

if size(h1,1) > 1
  error('lattice not found');
elseif size(h1,1) == 1
  Ms = [Ms; 2];
  hred_array = [hred_array; 1];
  hrem_array = [hrem_array; 0];
  P = 2;
  if iscell(r1l_z_array_cell)
    r1l_z = int64(ones(1,d));
    for t = 1:d
      for ii = 1:length(r1l_z_array_cell{t})
        r1l_z(t) = mod(r1l_z(t) * mod(r1l_z_array_cell{t}(ii),P), P);
      end
    end
  else
    r1l_z = mod(r1l_z_array_cell,P);
  end
  r1l_z_output = [r1l_z_output; r1l_z];
end

if max(Ms) > flintmax
  error('M too large to fit into double');
end
Ms = double(Ms);
r1l_z_output = double(r1l_z_output);

if ~check_iterative_MR1L(double(freq),Ms,r1l_z_output)
  error('final check failed');
end

end

