function [ val2, val2_truncerr, val2_aliaserr, val1, val1_truncerr, val1_aliaserr ] = testG3_mlfft2_run( freq, zs, Ms )
%TESTG3_MLFFT2_RUN Computes the error in using multiple rank-1 lattices for approximation on even hyperbolic crosses starting with exponential lattices for the full grid.

d = size(freq,2);
if d==1
  freq = unique(freq,'rows');
end

disp('Sampling f...');
drawnow;
f_sampled = [];
for ii=1:length(Ms)
  loc_samples = fct_Gp(3,zs(ii,:),Ms(ii));
  if ii > 1
    f_sampled = [f_sampled; loc_samples(2:end)];
  else
    f_sampled = [f_sampled; loc_samples];
  end
end
disp('Approximate reconstruction...');
drawnow;
optsmlfft.alg = 'direct';
[fhat_tilde,~,~] = multi_lattice_ifft_noniter_avg(f_sampled,freq,Ms,zs,1e-10,optsmlfft);
clear f_sampled

fprintf('Exact f_hat... ');
drawnow;
fhat = fct_Gp_hat(3,freq);

fct_G3_normL2_val = 1;
fct_G3_normA_val = fct_G3_normA(d);

val2 = sqrt( sum(sort(abs(fhat-fhat_tilde)).^2) + fct_G3_normL2_val^2 - sum(sort(abs(fhat)).^2)) / fct_G3_normL2_val;
val2_truncerr = sqrt(fct_G3_normL2_val^2 - sum(sort(abs(fhat)).^2)) / fct_G3_normL2_val;
val2_aliaserr = sqrt( sum(sort(abs(fhat-fhat_tilde).^2))) / fct_G3_normL2_val;

val1 = ( sum(sort(abs(fhat-fhat_tilde))) + fct_G3_normA_val - sum(sort(abs(fhat))) ) / fct_G3_normA_val;
val1_truncerr = ( fct_G3_normA_val - sum(sort(abs(fhat))) ) / fct_G3_normA_val;
val1_aliaserr = sum(sort(abs(fhat-fhat_tilde))) / fct_G3_normA_val;

if ((imag(val2) ~= 0) || (imag(val2_truncerr) ~= 0) || (imag(val2_aliaserr) ~= 0))
  error('error value should not have imaginary part');
end

if (val1 < 0) || (val1_truncerr < 0) || (val1_aliaserr < 0)
  error('error value should not be negative');
end

if ((val2 < 4e-8) && (size(freq,1) > 2000000))
  fprintf('val1: err = %.3e, %.3e, %.3e\n', val1, val1_truncerr, val1_aliaserr);
  fprintf('inacurrate values: err = %.3e, %.3e, %.3e\n', val2, val2_truncerr, val2_aliaserr);
  disp('error too small but index set too large for 128bit...');
  val2 = inf;
  val2_truncerr = inf;
  val2_aliaserr = inf;
elseif val2 < 4e-8
  fprintf('val1: err = %.3e, %.3e, %.3e\n', val1, val1_truncerr, val1_aliaserr);
  fprintf('inacurrate values: err = %.3e, %.3e, %.3e\n', val2, val2_truncerr, val2_aliaserr);
  disp('Exact f_hat 128bit...');
  fhat = fct_Gp_hat_mp(3,freq);
  fhat_tilde=mp(fhat_tilde,128);

  aliasterm_sq = sum(abs(fhat-fhat_tilde).^2);
  truncterm_sq = fct_Gp_norm(3)^2 - sum(abs(fhat).^2);
  val2 = sqrt( aliasterm_sq + truncterm_sq) / fct_Gp_norm(3);
%  val_truncerr = sqrt(fct_Gp_norm(p)^2 - sum(abs(fhat).^2)) / fct_Gp_norm(p);
  val2_truncerr = sqrt(truncterm_sq);
%  val_aliaserr = sqrt( sum(abs(fhat-fhat_tilde).^2)) / fct_Gp_norm(p);
  val2_aliaserr = sqrt(aliasterm_sq);

  val2 = double(val2);
  val2_truncerr = double(val2_truncerr);
  val2_aliaserr = double(val2_aliaserr);
% catch err
%   disp('Using DP code for error calculation...');
% val = sqrt( sum(abs(fhat-fhat_tilde).^2) + fct_Gp_norm(p)^2 - sum(abs(fhat).^2)) / fct_Gp_norm(p);
% val_truncerr = sqrt(fct_Gp_norm(p)^2 - sum(abs(fhat).^2)) / fct_Gp_norm(p);
% val_aliaserr = sqrt( sum(abs(fhat-fhat_tilde).^2)) / fct_Gp_norm(p);
end

end

