function [results] = test_mr1l_approx_rand_cbc(N,d,s_array,nrepeat)
%TEST_MR1L_APPROX_RAND_CBC Computes the number of samples in multiple rank-1 lattices constructed by Algorithm 2.1 for random frequency sets crosses starting with CBC construction.

filename = sprintf('test_mr1l_approx_rand_cbc_N%d_d%d.mat',N, d);

rng_state = rng;

results = {};

if max(s_array) > (2*N+1)^d
  error('max s > full cube');
end
  
for iis = 1:length(s_array)
 s = s_array(iis);
 for ir = 1:nrepeat
  result = struct();
  result.d = d;
  result.N = N;
  result.num_freq = s;
  
  freq = unique(randi(2*N+1, 2*s, d) - (N+1), 'rows', 'stable');
  while size(freq,1) < s
    freq = unique([freq; randi(2*N+1, 2*s, d) - (N+1)], 'rows', 'stable');
  end
  freq = freq(1:s,:);
  
  fprintf('N=%d, d=%d, ir=%d, s=%d, ', N, d, ir, s);

  [z,M] = lfft_gen_lattice_algo23mod( freq );
  fprintf('M=%d, ', M);

  result.z = z;
  result.M = M;
  [Ms,hred_array,hrem_array] = deterministic_mr1l_approx_get_lattice_sizes(freq,z,M);
  fprintf('numlat=%d, samples=%d\n', length(Ms), sum(Ms)-length(Ms)+1);
  result.Ms = Ms;
  result.hred_array = hred_array;
  result.hrem_array = hrem_array;
%[[Ms;0], [size(freq,1);hrem_array]];
  results{iis,ir} = result;
 end
 save(filename', 'N', 'd', 's_array', 'results', 'rng_state');
end

